import { Connections, IConnectable, ISecurityGroup, IVpc, SelectedSubnets } from 'aws-cdk-lib/aws-ec2';
import { CfnConnection } from 'aws-cdk-lib/aws-glue';
import { CfnWorkgroup } from 'aws-cdk-lib/aws-redshiftserverless';
import { Construct } from 'constructs';
import { RedshiftServerlessNamespace } from './redshift-serverless-namespace';
import { RedshiftServerlessWorkgroupProps } from './redshift-serverless-workgroup-props';
import { DataCatalogDatabase } from '../../../governance';
import { TrackedConstruct } from '../../../utils';
import { RedshiftData } from '../redshift/redshift-data';
/**
 * Create a Redshift Serverless Workgroup. A default namespace would be created if none is provided.
 *
 * @example
 * const workgroup = new dsf.consumption.RedshiftServerlessWorkgroup(this, "RedshiftWorkgroup", {
 *    name: "example-workgroup",
 *    namespace: new dsf.consumption.RedshiftServerlessNamespace(this, "RedshiftNamespace", {
 *      name: 'example-namespace',
 *      dbName: 'defaultdb',
 *    })
 * });
 */
export declare class RedshiftServerlessWorkgroup extends TrackedConstruct implements IConnectable {
    private static readonly DEFAULT_VPC_CIDR;
    private static readonly DEFAULT_VPC_PRIVATE_SUBNET_NAME;
    private static readonly DEFAULT_PORT;
    /**
     * The created Redshift Serverless Workgroup
     */
    readonly cfnResource: CfnWorkgroup;
    /**
     * Connections used by Workgroup security group. Used this to enable access from clients connecting to the workgroup
     */
    readonly connections: Connections;
    /**
     * The associated Redshift Serverless Namespace
     */
    readonly namespace: RedshiftServerlessNamespace;
    /**
     * The Glue Connection associated with the workgroup. This can be used by Glue ETL Jobs to read/write data from/to Redshift workgroup
     */
    readonly glueConnection: CfnConnection;
    /**
     * The primary EC2 Security Group associated with the Redshift Serverless Workgroup
     */
    readonly primarySecurityGroup: ISecurityGroup;
    /**
     * The VPC where the Redshift Serverless Workgroup is deployed
     */
    readonly vpc: IVpc;
    /**
     * The subnets where the Redshift Serverless Workgroup is deployed
     */
    readonly selectedSubnets: SelectedSubnets;
    private readonly removalPolicy;
    private redshiftData?;
    private port;
    constructor(scope: Construct, id: string, props: RedshiftServerlessWorkgroupProps);
    /**
     * Creates a new Glue data catalog database with a crawler using JDBC target type to connect to the Redshift Workgroup
     * @param id The CDK ID of the resource
     * @param catalogDbName The name of the Glue Database to create
     * @param pathToCrawl The path of Redshift tables to crawl @default `<databaseName>/public/%``
     * @returns The DataCatalogDatabase created
     */
    catalogTables(id: string, catalogDbName: string, pathToCrawl?: string): DataCatalogDatabase;
    /**
     * Create Glue Connection and modifies the Redshift Serverless Workgroup's security group to allow Glue access via self-referencing rule.
     * @param props The properties of the Redshift Serverless Workgroup
     * @returns The Glue Connection to the Redshift Serverless Workgroup
     */
    private createConnection;
    /**
     * Creates an instance of `RedshiftData` to send custom SQLs to the workgroup
     * @param createVpcEndpoint if set to true, create interface VPC endpoint for Redshift Data API
     * @returns `RedshiftData`
     */
    accessData(id: string, createVpcEndpoint?: boolean): RedshiftData;
    /**
     * Initialize Security Groups.
     * Convert `SecurityGroup[]` into array of security group ids.
     * Also create a primary security group that clients can use to add ingress rules.
     * @param props `RedshiftServerlessWorkgroupProps`
     * @returns `SecurityGroupDetails`
     */
    private initSecurityGroup;
    private initDefaultNetworking;
}
